# -*- coding: utf-8 -*-

# Copyright 2016 Resonai Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, unused-argument

"""
yabt Builders for tests
~~~~~~~~~~~~~~~~~~~~~~~

:author: Itamar Ostricher
"""


from ..extend import (
    PropType as PT, register_builder_sig, register_manipulate_target_hook)


register_builder_sig('DepTester', [('buildenv', PT.Target, None)])


@register_manipulate_target_hook('DepTester')
def dep_tester_manipulate_target(build_context, target):
    if target.props.buildenv:
        target.buildenv = target.props.buildenv



# -*- coding: utf-8 -*-

# Copyright 2020 Resonai Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, unused-argument

"""
yabt Go Builder
~~~~~~~~~~~~~~~

:author: Itamar Ostricher

TODO: libs, external libs
TODO: does this even work with non-flat source file tree??
"""
from ..config import YSETTINGS_FILE
from os import listdir, remove
from os.path import isfile, join, relpath

from yabt.docker import extend_runtime_params, format_docker_run_params
from ..artifact import ArtifactType as AT
from .dockerapp import build_app_docker_and_bin, register_app_builder_sig
from ..extend import (
    PropType as PT, register_build_func, register_builder_sig,
    register_manipulate_target_hook)
from ..logging import make_logger
from ..target_utils import split
from ..utils import link_files, link_node, rmtree, yprint

logger = make_logger(__name__)


register_app_builder_sig('GoApp', [('main', PT.Target)])


@register_manipulate_target_hook('GoApp')
def go_app_manipulate_target(build_context, target):
    logger.debug('Injecting {} to deps of {}',
                 target.props.base_image, target.name)
    target.deps.append(target.props.base_image)
    if target.props.main and target.props.main not in target.deps:
        logger.debug('Injecting {} to deps of {}',
                     target.props.main, target.name)
        target.deps.append(target.props.main)


@register_build_func('GoApp')
def go_app_builder(build_context, target):
    """Pack a Go binary as a Docker image with its runtime dependencies."""
    yprint(build_context.conf, 'Build GoApp', target)
    prog = build_context.targets[target.props.main]
    binary = list(prog.artifacts.get(AT.binary).keys())[0]
    entrypoint = ['/usr/src/bin/' + binary]
    build_app_docker_and_bin(
        build_context, target, entrypoint=entrypoint)


# Common Go builder signature terms
GO_SIG = [
    ('sources', PT.FileList),
    ('in_buildenv', PT.Target),
    ('go_package', PT.str, None),
    ('mod_file', PT.File, None),
    ('cmd_env', None),
]

register_builder_sig('GoProg', GO_SIG)


@register_manipulate_target_hook('GoProg')
def go_prog_manipulate_target(build_context, target):
    target.buildenv = target.props.in_buildenv


@register_build_func('GoProg')
def go_prog_builder(build_context, target):
    """Build a Go binary executable"""
    go_builder_internal(build_context, target, command='build')


register_builder_sig('GoTest', GO_SIG)


@register_manipulate_target_hook('GoTest')
def go_test_manipulate_target(build_context, target):
    target.tags.add('testable')
    target.buildenv = target.props.in_buildenv


@register_build_func('GoTest')
def go_test_builder(build_context, target):
    """Test a Go test"""
    go_builder_internal(build_context, target, command='test')


def rm_all_but_go_mod(workspace_dir):
    for fname in listdir(workspace_dir):
        if fname == 'go.mod':
            continue
        filepath = join(workspace_dir, fname)
        if isfile(filepath):
            remove(filepath)
        else:
            rmtree(filepath)


def go_builder_internal(build_context, target, command):
    """Build or test a Go binary executable.
    command is either build or test

    We link all go files source and all proto generated files into workspace.
    We generate a go.mod file in the workspace to make it the root of the
    go project.
    We create a go.mod file in proto dir with the same package name and add
    a "replace proto => ./proto" directive in the go.mod.
    We set the first dir in GOPATH to be yabtwork/go so that all downloaded
    packages are managed in the user machine and not inside the ephemeral
    docker.
    When we clean the workspace we make sure to keep the go.mod since it is new
    go build redownload all packages (can we solve this?)


    TODOs:
      - "replace proto => ./proto" is needed since the generated code import
        doesn't have the package before imports of other generated go files.
        See if there is another way to do it (understanding that can help us
        create a GoLib builder)
    """
    builder_name = target.builder_name
    yprint(build_context.conf, command, builder_name, target)
    workspace_dir = build_context.get_workspace(builder_name, target.name)
    go_package = (target.props.get('go_package') or
                  build_context.conf.get('go_package', None))
    go_mod_path = join(workspace_dir, 'go.mod')
    if not go_package:
        raise KeyError('Must specify go_package in {} common_conf '
                       'or on target'.format(YSETTINGS_FILE))

    # we leave the go.mod file otherwise the caching of downloaded packages
    # doesn't work
    rm_all_but_go_mod(workspace_dir)
    binary = join(*split(target.name))

    buildenv_workspace = build_context.conf.host_to_buildenv_path(
        workspace_dir)
    buildenv_sources = [join(buildenv_workspace, src)
                        for src in target.props.sources]
    if target.props.get('mod_file'):
        link_node(join(build_context.conf.project_root,
                       target.props.get('mod_file')),
                  go_mod_path)
    sources_to_link = list(target.props.sources)
    has_protos = False

    # Goging over all deps and pulling their sources.
    # TODO(eyal): This pull sources of all types which is most likely not
    # needed but not harming. We should revisit this in later iterations.
    for dep in build_context.generate_all_deps(target):
        sources_to_link.extend(dep.props.get('sources', []))
        artifact_map = dep.artifacts.get(AT.gen_go)
        if not artifact_map:
            continue
        has_protos = True
        for dst, src in artifact_map.items():
            target_file = join(workspace_dir, dst)
            link_node(join(build_context.conf.project_root, src), target_file)

    link_files(sources_to_link, workspace_dir, None, build_context.conf)

    download_cache_dir = build_context.conf.host_to_buildenv_path(
      build_context.conf.get_go_packages_path())

    gopaths = [download_cache_dir]
    user_gopath = (target.props.cmd_env or {}).get('GOPATH')
    # if user didn't provide GOPATH we assumes it is /go
    # TODO(eyal): A more correct behavior will be to check the GOPATH var
    # inside the docker then to assumes it is /go.
    # This code provides a way for the user to tell us what is the correct
    # GOPATH but if it come handy we should implement looking into the docker
    gopaths.append(user_gopath if user_gopath else '/go')
    build_cmd_env = {
        'XDG_CACHE_HOME': '/tmp/.cache',
    }
    build_cmd_env.update(target.props.cmd_env or {})
    build_cmd_env['GOPATH'] = ':'.join(gopaths)

    if not isfile(go_mod_path):
        build_context.run_in_buildenv(
          target.props.in_buildenv,
          ['go', 'mod', 'init', go_package],
          build_cmd_env,
          work_dir=buildenv_workspace)
    if has_protos:
        build_context.run_in_buildenv(
          target.props.in_buildenv,
          ['go', 'mod', 'edit', '-replace', 'proto=./proto'],
          build_cmd_env,
          work_dir=buildenv_workspace)
        if not isfile(join(workspace_dir, 'proto', 'go.mod')):
            build_context.run_in_buildenv(
              target.props.in_buildenv,
              ['go', 'mod', 'init', go_package],
              build_cmd_env,
              work_dir=join(buildenv_workspace, 'proto'))

    bin_file = join(buildenv_workspace, binary)
    build_cmd = ['go', command, '-o', bin_file] + buildenv_sources

    run_params = extend_runtime_params(
        target.props.runtime_params,
        build_context.walk_target_deps_topological_order(target),
        build_context.conf.runtime_params, True)

    build_context.run_in_buildenv(
      target.props.in_buildenv, build_cmd, build_cmd_env,
      run_params=format_docker_run_params(run_params),
      work_dir=buildenv_workspace)
    target.artifacts.add(
        AT.binary,
        relpath(join(workspace_dir, binary), build_context.conf.project_root),
        binary)



# -*- coding: utf-8 -*-

# Copyright 2016 Resonai Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
yabt Target extraction tests
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:author: Itamar Ostricher
"""


from os import path

import pytest

from .buildcontext import BuildContext
from .extend import (
    Plugin, PropType as PT, register_build_func, register_builder_sig,
    register_manipulate_target_hook)
from .target_extraction import args_to_props, extractor, handle_typed_args
from .target_utils import Target


@pytest.yield_fixture
def with_spam_builder_sig():
    Plugin.remove_builder('Spam')
    register_builder_sig('Spam', ['foo',
                                  ('bar', PT.str),
                                  ('cats', PT.StrList, None),
                                  ('cat_list', PT.File, None),
                                  ])
    yield


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_valid_positional_args():
    target = Target(builder_name='Spam')
    args_to_props(target, Plugin.builders['Spam'],
                  args=['my-spam', 'foo', 'bar'], kwargs={})
    assert target.props == {'name': 'my-spam', 'foo': 'foo', 'bar': 'bar',
                            'deps': None, 'cachable': True,
                            'cats': None, 'cat_list': None,
                            'build_params': None,
                            'license': None,
                            'packaging_params': None,
                            'policies': None,
                            'attempts': 1,
                            'runtime_params': None,
                            }


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_missing_one_pos_arg():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=['my-spam', 'foo'], kwargs={})
    assert ("Spam() missing 1 required positional argument: 'bar'"
            in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_missing_two_pos_args():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=['my-spam'], kwargs={})
    assert ("Spam() missing 2 required positional arguments: 'foo', 'bar'"
            in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_missing_all_pos_args():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=[], kwargs={})
    assert ("Spam() missing 3 required positional arguments: "
            "'name', 'foo', 'bar'" in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_valid_mix_pos_kwargs():
    target = Target(builder_name='Spam')
    args_to_props(target, Plugin.builders['Spam'],
                  args=['my-spam'], kwargs={'foo': 'foo', 'bar': 'bar'})
    assert target.props == {'name': 'my-spam', 'foo': 'foo', 'bar': 'bar',
                            'deps': None, 'cachable': True,
                            'cats': None, 'cat_list': None,
                            'build_params': None,
                            'license': None,
                            'packaging_params': None,
                            'policies': None,
                            'attempts': 1,
                            'runtime_params': None,
                            }


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_mix_pos_kwargs_missing_pos_arg():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=['my-spam'], kwargs={'bar': 'bar'})
    assert ("Spam() missing 1 required positional argument: 'foo'"
            in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_mix_pos_kwargs_dup_arg():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=['my-spam'], kwargs={'name': 'my-spam'})
    assert ("Spam() got multiple values for argument 'name'"
            in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_mix_pos_kwargs_unexpected_kwarg():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=['my-spam'], kwargs={'wtf': 'wait-for-it'})
    assert ("Spam() got an unexpected keyword argument 'wtf'"
            in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_build_sig_args_to_props_too_many_pos_args():
    target = Target(builder_name='Spam')
    with pytest.raises(TypeError) as excinfo:
        args_to_props(target, Plugin.builders['Spam'],
                      args=['my-spam', 'foo', 'bar', None, None, None, None,
                            None, None, None, None, None, 'w00t', '14th'],
                      kwargs={})
    assert ('Spam() takes from 3 to 13 positional arguments, but 14 were given'
            in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_target_extraction_bad_target_names(basic_conf):
    for bad_name in ['.', 'bad:name', ':', 'bad@name', '@', '**', 'foo/bar']:
        target = Target(builder_name='Spam')
        args_to_props(target, Plugin.builders['Spam'],
                      args=[bad_name, 'foo', 'bar'], kwargs={})
        with pytest.raises(ValueError) as excinfo:
            handle_typed_args(target, Plugin.builders['Spam'], 'spams')
        assert ("Invalid target name: `{}'".format(bad_name)
                in str(excinfo.value))


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_typed_args_not_str():
    target = Target(builder_name='Spam')
    args_to_props(target, Plugin.builders['Spam'],
                  args=['my-spam', 'foo', 123], kwargs={})
    with pytest.raises(TypeError) as excinfo:
        handle_typed_args(target, Plugin.builders['Spam'], 'spams')
    assert 'bar: got `123`, expected string value' in str(excinfo.value)


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_typed_args_valid_defaults():
    target = Target(builder_name='Spam')
    args_to_props(target, Plugin.builders['Spam'],
                  args=['my-spam', 'foo', 'bar'], kwargs={})
    handle_typed_args(target, Plugin.builders['Spam'], 'spams')
    assert target.props == {'name': 'spams:my-spam', 'foo': 'foo',
                            'bar': 'bar', 'deps': [], 'cachable': True,
                            'cats': [], 'cat_list': None,
                            'build_params': {},
                            'license': [],
                            'packaging_params': {},
                            'policies': [],
                            'attempts': 1,
                            'runtime_params': {},
                            }


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_typed_args_valid_non_default():
    target = Target(builder_name='Spam')
    args_to_props(target, Plugin.builders['Spam'],
                  args=['my-spam', 'foo', 'bar', ('my cat', 'other cat'),
                        '../lists/from-vet-4', '../hams:my-ham'],
                  kwargs={})
    handle_typed_args(target, Plugin.builders['Spam'], 'spams')
    assert target.props == {'name': 'spams:my-spam', 'foo': 'foo',
                            'bar': 'bar', 'deps': ['hams:my-ham'],
                            'cachable': True, 'cats': ['my cat', 'other cat'],
                            'cat_list': path.join('lists', 'from-vet-4'),
                            'build_params': {},
                            'license': [],
                            'packaging_params': {},
                            'policies': [],
                            'attempts': 1,
                            'runtime_params': {},
                            }


@pytest.mark.usefixtures('with_spam_builder_sig')
def test_file_arg_from_project_root():
    target = Target(builder_name='Spam')
    args_to_props(target, Plugin.builders['Spam'],
                  args=['my-spam', 'foo', 'bar'],
                  kwargs={'cat_list': '//lists/from-vet-4'})
    handle_typed_args(target, Plugin.builders['Spam'], 'spams')
    assert target.props == {'name': 'spams:my-spam', 'foo': 'foo',
                            'bar': 'bar', 'deps': [], 'cachable': True,
                            'cats': [],
                            'cat_list': path.join('lists', 'from-vet-4'),
                            'build_params': {},
                            'license': [],
                            'packaging_params': {},
                            'policies': [],
                            'attempts': 1,
                            'runtime_params': {},
                            }



# -*- coding: utf-8 -*-

# Copyright 2016 Resonai Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
yabt target extraction module
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

:author: Itamar Ostricher
"""


from numbers import Number
import types

from ostrich.utils.collections import listify

from .buildfile_utils import to_build_module
from .extend import Builder, Empty, Plugin, PropType as PT
from .logging import make_logger
from .target_utils import norm_name, split_build_module, Target, validate_name
from .utils import norm_proj_path


logger = make_logger(__name__)


def format_num_positional_arguments(builder: Builder):
    min_args = builder.min_positional_args
    max_args = len(builder.sig)
    if min_args < max_args:
        return 'from {} to {} positional arguments'.format(min_args, max_args)
    # TODO(itamar): use inflect()
    return '{} positional argument{}'.format(min_args,
                                             's' if min_args > 1 else '')


def args_to_props(target: Target, builder: Builder, args: list, kwargs: dict):
    """Convert build file `args` and `kwargs` to `target` props.

    Use builder signature to validate builder usage in build-file, raising
    appropriate exceptions on signature-mismatches.

    Use builder signature default values to assign props values to args that
    were not passed in the build-file call.

    This function handles only the arg/kwargs-to-prop assignment, including
    default values when necessary. When it returns, if no exception was raised,
    it is guaranteed that `target.props` contains all args defined in the
    builder registered signature, with values taken either from the build-file
    call, or from default values provided in the signature.

    Specifically, this function DOES NOT do anything about the arg types
    defined in the builder signature.

    :raise TypeError: On signature-call mismatch.
    """
    if len(args) > len(builder.sig):
        # too many positional arguments supplied - say how many we can take
        raise TypeError('{}() takes {}, but {} were given'
                        .format(target.builder_name,
                                format_num_positional_arguments(builder),
                                len(args)))
    # read given args into the matching props according to the signature
    for arg_name, value in zip(builder.sig.keys(), args):
        target.props[arg_name] = value
    # read given kwargs into the named props, asserting matching sig arg names
    for arg_name, value in kwargs.items():
        if arg_name not in builder.sig:
            raise TypeError("{}() got an unexpected keyword argument '{}'"
                            .format(target.builder_name, arg_name))
        if arg_name in target.props:
            raise TypeError("{}() got multiple values for argument '{}'"
                            .format(target.builder_name, arg_name))
        target.props[arg_name] = value
    # go over signature args, assigning default values to anything that wasn't
    # assigned from args / kwargs, making sure no positional args are missing
    missing_args = []
    for arg_name, sig_spec in builder.sig.items():
        if arg_name not in target.props:
            if sig_spec.default == Empty:
                missing_args.append(arg_name)
            else:
                target.props[arg_name] = sig_spec.default
    if missing_args:
        # not enough positional arguments supplied - say which
        # TODO(itamar): match Python's error more closely (last "and "):
        # foo() missing 3 required positional arguments: 'a', 'b', and 'c'
        # TODO(itamar): use inflect
        raise TypeError('{}() missing {} required positional argument{}: {}'
                        .format(target.builder_name, len(missing_args),
                                's' if len(missing_args) > 1 else '',
                                ', '.join("'{}'".format(arg)
                                          for arg in missing_args)))
    logger.debug('Got props for target: {}', target)


def handle_typed_args(target, builder, build_module):

    def assert_type(arg_name, value, class_or_type_or_tuple, type_name):
        if value is None or isinstance(value, class_or_type_or_tuple):
            return value
        raise TypeError('{}: got `{}`, expected {} value'
                        .format(arg_name, value, type_name))

    def handle_target_name(arg_name, value):
        return '{}:{}'.format(
            build_module,
            validate_name(assert_type(arg_name, value, str, 'string')))

    def handle_target_ref(arg_name, value):
        return norm_name(build_module,
                         assert_type(arg_name, value, str, 'string'))

    def handle_file(arg_name, value):
        if assert_type(arg_name, value, str, 'filename') is not None:
            return norm_proj_path(value, build_module)

    for arg_name, value in target.props.items():
        arg_type = builder.sig[arg_name].type
        if arg_type == PT.str:
            assert_type(arg_name, value, str, 'string')
        elif arg_type == PT.numeric:
            assert_type(arg_name, value, Number, 'numeric')
        elif arg_type == PT.list:
            target.props[arg_name] = listify(value)
        elif arg_type == PT.dict:
            # replace `None` with empty dict
            assert_type(arg_name, value, dict, 'dict')
            if value is None:
                target.props[arg_name] = dict()
        elif arg_type == PT.StrList:
            target.props[arg_name] = [
                assert_type(arg_name, val, str, 'string')
                for val in listify(value)]
        elif arg_type == PT.TargetName:
            target.props[arg_name] = handle_target_name(arg_name, value)
        elif arg_type == PT.Target:
            target.props[arg_name] = (
                None if value is None else handle_target_ref(arg_name, value))
        elif arg_type == PT.TargetList:
            target.props[arg_name] = [
                handle_target_ref(arg_name, val) for val in listify(value)]
        elif arg_type == PT.File:
            target.props[arg_name] = handle_file(arg_name, value)
        elif arg_type == PT.FileList:
            target.props[arg_name] = [
                handle_file(arg_name, val) for val in listify(value)]


def extractor(
        builder_name: str, builder: Builder, build_file_path: str,
        build_context) -> types.FunctionType:
    """Return a target extraction function for a specific builder and a
       specific build file."""
    build_module = to_build_module(build_file_path, build_context.conf)

    def extract_target(*args, **kwargs):
        """The actual target extraction function that is executed when any
           builder function is called in a build file."""
        target = Target(builder_name=builder_name)
        # convert args/kwargs to target.props and handle arg types
        args_to_props(target, builder, args, kwargs)
        raw_name = target.props.name
        handle_typed_args(target, builder, build_module)
        logger.debug('Extracting target: {}', target)
        # promote the `name` and `deps` from props to the target instance
        target.name = target.props.pop('name')
        target.deps = target.props.pop('deps', [])
        if target.deps:
            logger.debug('Got deps for target "{0.name}": {0.deps}', target)
        # invoke builder hooks on extracted target
        for hook_name, hook in Plugin.get_hooks_for_builder(builder_name):
            logger.debug('About to invoke hook {} on target {}',
                         hook_name, target)
            hook(build_context, target)
        # save the target in the build context
        build_context.register_target(target)
        logger.debug('Registered {}', target)

    return extract_target



# -*- coding: utf-8 -*-

# Copyright 2016 Resonai Ltd. All rights reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# pylint: disable=invalid-name, unused-argument

"""
yabt File group builder
~~~~~~~~~~~~~~~~~~~~~~~

:author: Itamar Ostricher
"""


from ..artifact import ArtifactType as AT
from ..extend import (
    PropType as PT, register_build_func, register_builder_sig,
    register_manipulate_target_hook)
from ..utils import yprint


register_builder_sig(
    'FileGroup',
    [('files', PT.FileList, None),
     ('kind', PT.str, 'app'),
     ])


@register_build_func('FileGroup')
def file_group_builder(build_context, target):
    yprint(build_context.conf, 'Build FileGroup', target)
    kind = getattr(AT, target.props.kind)
    target.artifacts.extend(kind, target.props.files)
